from typing import Any

from cfgs import App


_config_file = App("alga").config.open("config.json")
_latest_version = 2


def get() -> dict[str, Any]:
    config = _config_file.contents

    if config.setdefault("version", _latest_version) < _latest_version:
        config = migrate(config)
        write(config)

    config.setdefault("tvs", {})

    return config


def migrate(config: dict[str, Any]) -> dict[str, Any]:
    if config["version"] == 1:
        config = {
            "version": 2,
            "default_tv": "default",
            "tvs": {
                "default": {
                    "hostname": config["hostname"],
                    "key": config["key"],
                    "mac": config["mac"],
                }
            },
        }

    return config


def write(config: dict[str, Any]) -> None:
    _config_file.contents = config
    _config_file.write()



import json
from unittest.mock import MagicMock

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app


runner = CliRunner()


def test_without_data(faker: Faker, mock_request: MagicMock) -> None:
    path = faker.pystr()
    return_value = faker.pystr()
    mock_request.return_value = return_value

    result = runner.invoke(app, ["adhoc", path])

    mock_request.assert_called_once_with(path)
    assert result.exit_code == 0
    assert result.stdout == f"{return_value}\n"


def test_with_data(faker: Faker, mock_request: MagicMock) -> None:
    path = faker.pystr()
    data = faker.pydict(allowed_types=[str, float, int])
    return_value = faker.pystr()
    mock_request.return_value = return_value

    result = runner.invoke(app, ["adhoc", path, json.dumps(data)])

    mock_request.assert_called_once_with(path, data)
    assert result.exit_code == 0
    assert result.stdout == f"{return_value}\n"



import json
from unittest.mock import MagicMock, call, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app
from alga.types import App


runner = CliRunner()


def test_current(faker: Faker, mock_request: MagicMock) -> None:
    app_id = faker.pystr()
    mock_request.return_value = {"appId": app_id}

    result = runner.invoke(app, ["app", "current"])

    mock_request.assert_called_once_with(
        "ssap://com.webos.applicationManager/getForegroundAppInfo"
    )
    assert result.exit_code == 0
    assert result.stdout == f"The current app is {app_id}\n"


def test_close(faker: Faker, mock_request: MagicMock) -> None:
    app_id = faker.pystr()

    result = runner.invoke(app, ["app", "close", app_id])

    mock_request.assert_called_once_with("ssap://system.launcher/close", {"id": app_id})
    assert result.exit_code == 0
    assert result.stdout == ""


def test_launch_without_data(faker: Faker, mock_request: MagicMock) -> None:
    app_id = faker.pystr()

    result = runner.invoke(app, ["app", "launch", app_id])

    mock_request.assert_called_once_with(
        "ssap://system.launcher/launch", {"id": app_id}
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_launch_with_data(faker: Faker, mock_request: MagicMock) -> None:
    app_id = faker.pystr()
    data = faker.pydict()

    result = runner.invoke(app, ["app", "launch", app_id, json.dumps(data)])

    mock_request.assert_called_once_with(
        "ssap://system.launcher/launch", {"id": app_id} | data
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_list(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "apps": [
            {"id": faker.pystr(), "title": faker.pystr()},
            {"id": faker.pystr(), "title": faker.pystr()},
            {"id": faker.pystr(), "title": faker.pystr()},
        ]
    }
    mock_request.return_value = return_value

    result = runner.invoke(app, ["app", "list"])

    mock_request.assert_called_once_with("ssap://com.webos.applicationManager/listApps")
    assert result.exit_code == 0

    splitted_output = result.stdout.split("\n")
    assert len(splitted_output) == (
        3  # table header
        + 3  # apps
        + 1  # table footer
        + 1  # trailing newline
    )
    assert splitted_output[3:6] == sorted(splitted_output[3:6])


def test_info(faker: Faker, mock_request: MagicMock) -> None:
    app_id = faker.pystr()
    app_info = faker.pystr()
    mock_request.return_value = {"appInfo": app_info}

    result = runner.invoke(app, ["app", "info", app_id])

    mock_request.assert_called_once_with(
        "ssap://com.webos.applicationManager/getAppInfo", {"id": app_id}
    )
    assert result.exit_code == 0
    assert result.stdout == f"{app_info}\n"


def test_pick(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "apps": [
            {"id": faker.pystr(), "title": faker.pystr()},
            {"id": faker.pystr(), "title": faker.pystr()},
            {"id": faker.pystr(), "title": faker.pystr()},
        ]
    }
    mock_request.return_value = return_value
    first_app = return_value["apps"][0]

    with patch("alga.cli_app.pzp") as mock_pzp:
        mock_pzp.return_value = App(first_app)

        result = runner.invoke(app, ["app", "pick"])

    mock_request.assert_has_calls(
        [
            call("ssap://com.webos.applicationManager/listApps"),
            call("ssap://system.launcher/launch", {"id": first_app["id"]}),
        ]
    )
    assert result.exit_code == 0



from typing import Optional

def get_mac_address(
    interface: Optional[str] = None,
    ip: Optional[str] = None,
    ip6: Optional[str] = None,
    hostname: Optional[str] = None,
    network_request: bool = True,
) -> Optional[str]: ...



from collections.abc import Iterator
from unittest.mock import MagicMock, patch

import pytest


@pytest.fixture
def mock_request() -> Iterator[MagicMock]:
    with patch("alga.client.request") as mocked:
        yield mocked


@pytest.fixture
def mock_input() -> Iterator[MagicMock]:
    with patch("alga.cli_remote._input_connection") as mocked:
        yield mocked.return_value.__enter__.return_value.send


@pytest.fixture
def mock_config() -> Iterator[MagicMock]:
    with patch("alga.config.get") as mocked:
        yield mocked


@pytest.fixture
def mock_config_file() -> Iterator[MagicMock]:
    with patch("alga.config._config_file") as mocked:
        yield mocked



from typer.testing import CliRunner

from alga import __version__
from alga.__main__ import app


runner = CliRunner()


def test_version() -> None:
    result = runner.invoke(app, ["version"])
    assert result.exit_code == 0
    assert result.stdout == f"alga version {__version__}\n"



from unittest.mock import MagicMock, patch

from faker import Faker

from alga import config


def test_get_sets_defaults(mock_config_file: MagicMock) -> None:
    mock_config_file.contents = {}

    cfg = config.get()

    assert cfg == {"version": 2, "tvs": {}}


def test_get_does_not_override_version(
    mock_config_file: MagicMock, faker: Faker
) -> None:
    version = faker.pyint()
    mock_config_file.contents = {"version": version}

    cfg = config.get()

    assert cfg == {"version": version, "tvs": {}}


def test_get_returns_data(mock_config_file: MagicMock, faker: Faker) -> None:
    data = {"version": faker.pyint(), "tvs": {}} | faker.pydict()
    mock_config_file.contents = data

    cfg = config.get()

    assert cfg == data


def test_get_calls_migrate(mock_config_file: MagicMock) -> None:
    mock_config_file.contents = {"version": 1}

    with patch("alga.config.migrate") as mock_migrate:
        config.get()

    mock_migrate.assert_called_once()


def test_write(mock_config_file: MagicMock, faker: Faker) -> None:
    data = faker.pydict()

    config.write(data)

    mock_config_file.write.assert_called_once()
    assert mock_config_file.contents == data


def test_migrate_v1_to_v2(faker: Faker) -> None:
    hostname, key, mac = faker.pystr(), faker.pystr(), faker.pystr()
    v1_config = {"version": 1, "hostname": hostname, "key": key, "mac": mac}

    assert config.migrate(v1_config) == {
        "version": 2,
        "default_tv": "default",
        "tvs": {"default": {"hostname": hostname, "key": key, "mac": mac}},
    }



from unittest.mock import MagicMock, call, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app
from alga.types import Channel


runner = CliRunner()


def test_current(faker: Faker, mock_request: MagicMock) -> None:
    channel_name = faker.pystr()
    channel_number = faker.pyint()
    mock_request.return_value = {
        "channelName": channel_name,
        "channelNumber": channel_number,
    }

    result = runner.invoke(app, ["channel", "current"])

    mock_request.assert_called_once_with("ssap://tv/getCurrentChannel")
    assert result.exit_code == 0
    assert (
        result.stdout == f"The current channel is {channel_name} ({channel_number})\n"
    )


def test_up(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["channel", "up"])

    mock_request.assert_called_once_with("ssap://tv/channelUp")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_down(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["channel", "down"])

    mock_request.assert_called_once_with("ssap://tv/channelDown")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_set_channel_id(faker: Faker, mock_request: MagicMock) -> None:
    channel_id = faker.pystr()

    result = runner.invoke(app, ["channel", "set", channel_id])

    mock_request.assert_called_once_with(
        "ssap://tv/openChannel", {"channelId": channel_id}
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_set_channel_number(faker: Faker, mock_request: MagicMock) -> None:
    channel_number = str(faker.pyint())
    channel_id = faker.pystr()

    mock_request.return_value = {
        "channelList": [{"channelId": channel_id, "channelNumber": channel_number}]
    }

    result = runner.invoke(app, ["channel", "set", channel_number])

    mock_request.assert_called_with("ssap://tv/openChannel", {"channelId": channel_id})
    assert result.exit_code == 0
    assert result.stdout == ""


def test_list(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "channelList": [
            {
                "channelNumber": f"{faker.pyint()}",
                "channelTypeId": faker.random_element([1, 2]),
                "channelName": faker.pystr(),
            },
            {
                "channelNumber": f"{faker.pyint()}",
                "channelTypeId": faker.random_element([1, 2]),
                "channelName": faker.pystr(),
            },
            {
                "channelNumber": f"{faker.pyint()}",
                "channelTypeId": faker.random_element([1, 2]),
                "channelName": faker.pystr(),
            },
        ]
    }
    mock_request.return_value = return_value

    result = runner.invoke(app, ["channel", "list"])

    mock_request.assert_called_once_with("ssap://tv/getChannelList")
    assert result.exit_code == 0

    splitted_output = result.stdout.split("\n")
    assert len(splitted_output) == (
        3  # table header
        + 3  # channels
        + 1  # table footer
        + 1  # trailing newline
    )


def test_pick(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "channelList": [
            {
                "channelId": faker.pystr(),
                "channelName": faker.pystr(),
                "channelNumber": f"{faker.pyint()}",
            },
            {
                "channelId": faker.pystr(),
                "channelName": faker.pystr(),
                "channelNumber": f"{faker.pyint()}",
            },
            {
                "channelId": faker.pystr(),
                "channelName": faker.pystr(),
                "channelNumber": f"{faker.pyint()}",
            },
        ]
    }
    mock_request.return_value = return_value
    first_channel = return_value["channelList"][0]

    with patch("alga.cli_channel.pzp") as mock_pzp:
        mock_pzp.return_value = Channel(first_channel)

        result = runner.invoke(app, ["channel", "pick"])

    mock_request.assert_has_calls(
        [
            call("ssap://tv/getChannelList"),
            call("ssap://tv/openChannel", {"channelId": first_channel["channelId"]}),
        ]
    )
    assert result.exit_code == 0



from typing import Annotated, Optional

from typer import Option, Typer

from alga import (
    cli_adhoc,
    cli_app,
    cli_channel,
    cli_input,
    cli_media,
    cli_power,
    cli_remote,
    cli_sound_output,
    cli_tv,
    cli_version,
    cli_volume,
    state,
)


def global_options(
    tv: Annotated[
        Optional[str], Option(help="Specify which TV the command should be sent to")
    ] = None,
) -> None:
    state.tv_id = tv


app = Typer(no_args_is_help=True, callback=global_options)
app.add_typer(cli_adhoc.app)
app.add_typer(cli_app.app, name="app")
app.add_typer(cli_channel.app, name="channel")
app.add_typer(cli_input.app, name="input")
app.add_typer(cli_media.app, name="media")
app.add_typer(cli_power.app, name="power")
app.add_typer(cli_remote.app, name="remote")
app.add_typer(cli_sound_output.app, name="sound-output")
app.add_typer(cli_tv.app, name="tv")
app.add_typer(cli_version.app)
app.add_typer(cli_volume.app, name="volume")


if __name__ == "__main__":
    app()



from faker import Faker

from alga.types import App, Channel, InputDevice, SoundOutputDevice


def test_app(faker: Faker) -> None:
    id_, title = faker.pystr(), faker.pystr()
    app = App({"id": id_, "title": title})

    assert str(app) == f"{title} ({id_})"


def test_channel(faker: Faker) -> None:
    number, name = faker.pystr(), faker.pystr()
    channel = Channel(
        {"channelId": faker.pystr(), "channelNumber": number, "channelName": name}
    )

    assert str(channel) == f"{number}: {name}"


def test_input_device(faker: Faker) -> None:
    id_, name = faker.pystr(), faker.pystr()
    input_device = InputDevice({"id": id_, "label": name})

    assert str(input_device) == f"{name} ({id_})"


def test_sound_output_device(faker: Faker) -> None:
    name = faker.pystr()
    sound_output_device = SoundOutputDevice(faker.pystr(), name)

    assert str(sound_output_device) == name



from unittest.mock import MagicMock

from typer.testing import CliRunner

from alga.__main__ import app


runner = CliRunner()


def test_play(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["media", "play"])

    mock_request.assert_called_once_with("ssap://media.controls/play")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_pause(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["media", "pause"])

    mock_request.assert_called_once_with("ssap://media.controls/pause")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_stop(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["media", "stop"])

    mock_request.assert_called_once_with("ssap://media.controls/stop")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_fast_forward(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["media", "fast-forward"])

    mock_request.assert_called_once_with("ssap://media.controls/fastForward")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_rewind(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["media", "rewind"])

    mock_request.assert_called_once_with("ssap://media.controls/rewind")
    assert result.exit_code == 0
    assert result.stdout == ""



import json
from typing import Annotated, Optional

from pzp import pzp
from rich import print
from rich.console import Console
from rich.table import Table
from typer import Argument, Typer

from alga import client
from alga.types import App


app = Typer(no_args_is_help=True, help="Apps installed on the TV")


@app.command()
def close(app_id: Annotated[str, Argument()]) -> None:
    """Close the provided app"""

    client.request("ssap://system.launcher/close", {"id": app_id})


@app.command()
def current() -> None:
    """Get the current app"""

    response = client.request(
        "ssap://com.webos.applicationManager/getForegroundAppInfo"
    )
    print(f"The current app is [bold]{response['appId']}[/bold]")


@app.command()
def info(app_id: str) -> None:
    """Show info about specific app"""

    response = client.request(
        "ssap://com.webos.applicationManager/getAppInfo", {"id": app_id}
    )

    print(response["appInfo"])


@app.command()
def launch(
    app_id: Annotated[str, Argument()],
    data: Annotated[Optional[str], Argument()] = None,
) -> None:
    """Launch an app"""

    payload = {"id": app_id}
    if data:
        payload.update(json.loads(data))
    client.request("ssap://system.launcher/launch", payload)


@app.command()
def list() -> None:
    """List installed apps"""

    response = client.request("ssap://com.webos.applicationManager/listApps")

    table = Table()
    table.add_column("Name")
    table.add_column("ID")

    all_apps = []
    for app in response["apps"]:
        all_apps.append([app["title"], app["id"]])

    for row in sorted(all_apps):
        table.add_row(*row)

    console = Console()
    console.print(table)


@app.command()
def pick() -> None:
    """Show picker for selecting an app."""

    response = client.request("ssap://com.webos.applicationManager/listApps")
    apps = []

    for app in response["apps"]:
        apps.append(App(app))

    app = pzp(candidates=apps, fullscreen=False, layout="reverse")
    if app:
        client.request("ssap://system.launcher/launch", {"id": app.id_})



from unittest.mock import MagicMock

import pytest
from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app


runner = CliRunner()


def test_up(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["volume", "up"])

    mock_request.assert_called_once_with("ssap://audio/volumeUp")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_down(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["volume", "down"])

    mock_request.assert_called_once_with("ssap://audio/volumeDown")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_set(faker: Faker, mock_request: MagicMock) -> None:
    volume = faker.pyint()

    result = runner.invoke(app, ["volume", "set", f"{volume}"])

    mock_request.assert_called_once_with("ssap://audio/setVolume", {"volume": volume})
    assert result.exit_code == 0
    assert result.stdout == ""


@pytest.mark.parametrize(
    ["muted", "version"], [[True, "old"], [False, "old"], [True, "new"], [False, "new"]]
)
def test_get(faker: Faker, mock_request: MagicMock, muted: bool, version: str) -> None:
    volume = faker.pyint()
    if version == "old":
        mock_request.return_value = {"volume": volume, "muted": muted}
    else:
        mock_request.return_value = {
            "volumeStatus": {"volume": volume, "muteStatus": muted}
        }

    result = runner.invoke(app, ["volume", "get"])

    mock_request.assert_called_once_with("ssap://audio/getVolume")
    assert result.exit_code == 0

    muted_text = ""
    if not muted:
        muted_text = "not "

    assert (
        result.stdout
        == f"Volume is currently set to {volume} and is currently {muted_text}muted\n"
    )


def test_mute(faker: Faker, mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["volume", "mute"])

    mock_request.assert_called_once_with("ssap://audio/setMute", {"mute": True})
    assert result.exit_code == 0
    assert result.stdout == ""


def test_unmute(faker: Faker, mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["volume", "unmute"])

    mock_request.assert_called_once_with("ssap://audio/setMute", {"mute": False})
    assert result.exit_code == 0
    assert result.stdout == ""



from typer import Typer

from alga import client


app = Typer(no_args_is_help=True, help="Control the playing media")


@app.command()
def fast_forward() -> None:
    """Fast forward media"""

    client.request("ssap://media.controls/fastForward")


@app.command()
def pause() -> None:
    """Pause media"""

    client.request("ssap://media.controls/pause")


@app.command()
def play() -> None:
    """Play media"""

    client.request("ssap://media.controls/play")


@app.command()
def rewind() -> None:
    """Rewind media"""

    client.request("ssap://media.controls/rewind")


@app.command()
def stop() -> None:
    """Stop media"""

    client.request("ssap://media.controls/stop")



from unittest.mock import MagicMock, call, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app
from alga.types import InputDevice


runner = CliRunner()


def test_set(faker: Faker, mock_request: MagicMock) -> None:
    input_name = faker.pystr()

    result = runner.invoke(app, ["input", "set", input_name])

    mock_request.assert_called_once_with(
        "ssap://tv/switchInput", {"inputId": input_name}
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_list(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "devices": [
            {"id": faker.pystr(), "label": faker.pystr()},
            {"id": faker.pystr(), "label": faker.pystr()},
            {"id": faker.pystr(), "label": faker.pystr()},
        ]
    }
    mock_request.return_value = return_value

    result = runner.invoke(app, ["input", "list"])

    mock_request.assert_called_once_with("ssap://tv/getExternalInputList")
    assert result.exit_code == 0

    splitted_output = result.stdout.split("\n")
    assert len(splitted_output) == (
        3  # table header
        + 3  # inputs
        + 1  # table footer
        + 1  # trailing newline
    )


def test_pick(faker: Faker, mock_request: MagicMock) -> None:
    return_value = {
        "devices": [
            {"id": faker.pystr(), "label": faker.pystr()},
            {"id": faker.pystr(), "label": faker.pystr()},
            {"id": faker.pystr(), "label": faker.pystr()},
        ]
    }
    mock_request.return_value = return_value
    first_input = return_value["devices"][0]

    with patch("alga.cli_input.pzp") as mock_pzp:
        mock_pzp.return_value = InputDevice(first_input)

        result = runner.invoke(app, ["input", "pick"])

    mock_request.assert_has_calls(
        [
            call("ssap://tv/getExternalInputList"),
            call("ssap://tv/switchInput", {"inputId": first_input["id"]}),
        ]
    )
    assert result.exit_code == 0



from rich import print
from typer import Typer

from alga import __version__


app = Typer()


@app.command()
def version() -> None:
    """Print Alga version"""

    print(f"alga version [bold]{__version__}[/bold]")



from typing import Any

class App:
    config: Directory

    def __init__(self, name: str, format: str = ...) -> None: ...

class Directory:
    def __init__(self, home: str, dirs: str, format: str) -> None: ...
    def open(self, filename: str | None = None) -> File: ...

class File:
    contents: dict[str, Any]
    def write(self) -> None: ...



from unittest.mock import MagicMock, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app


runner = CliRunner()


def test_off(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["power", "off"])

    mock_request.assert_called_once_with("ssap://system/turnOff")
    assert result.exit_code == 0
    assert result.stdout == ""


def test_screen_off(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["power", "screen-off"])

    mock_request.assert_called_once_with(
        "ssap://com.webos.service.tvpower/power/turnOffScreen"
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_screen_on(mock_request: MagicMock) -> None:
    result = runner.invoke(app, ["power", "screen-on"])

    mock_request.assert_called_once_with(
        "ssap://com.webos.service.tvpower/power/turnOnScreen"
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_on(mock_config: MagicMock, faker: Faker) -> None:
    mac_address = faker.pystr()
    mock_config.return_value = {
        "default_tv": "default",
        "tvs": {"default": {"mac": mac_address}},
    }

    with patch("alga.cli_power.send_magic_packet") as mock_send_magic_packet:
        result = runner.invoke(app, ["power", "on"])

    mock_send_magic_packet.assert_called_once_with(mac_address)
    assert result.exit_code == 0
    assert result.stdout == ""



from socket import AF_INET, SOCK_DGRAM, SOCK_STREAM, gaierror
from unittest.mock import MagicMock, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app
from alga.cli_tv import _ip_from_hostname


runner = CliRunner()


@patch("alga.cli_tv.getaddrinfo")
def test_ip_from_hostname(mock_getaddrinfo: MagicMock, faker: Faker) -> None:
    hostname = faker.hostname()
    ip_address = faker.ipv4()

    mock_getaddrinfo.return_value = [
        (AF_INET, SOCK_DGRAM, 17, "", (ip_address, 0)),
        (AF_INET, SOCK_STREAM, 6, "", (ip_address, 0)),
    ]

    result = _ip_from_hostname(hostname)

    mock_getaddrinfo.assert_called_once_with(host=hostname, port=None)
    assert result == ip_address


@patch("alga.cli_tv.getaddrinfo")
def test_ip_from_hostname_not_found(mock_getaddrinfo: MagicMock, faker: Faker) -> None:
    hostname = faker.hostname()
    mock_getaddrinfo.side_effect = gaierror(
        "[Errno 8] nodename nor servname provided, or not known"
    )

    result = _ip_from_hostname(hostname)

    mock_getaddrinfo.assert_called_once_with(host=hostname, port=None)
    assert result is None


@patch("alga.cli_tv._ip_from_hostname")
def test_add_ip_not_found(mock_ip_from_hostname: MagicMock, faker: Faker) -> None:
    hostname = faker.hostname()
    mock_ip_from_hostname.return_value = None

    result = runner.invoke(app, ["tv", "add", "name", hostname])

    mock_ip_from_hostname.assert_called_once_with(hostname)
    assert result.exit_code == 1
    assert (
        result.stdout.replace("\n", "")
        == f"Could not find any host by the name '{hostname}'. Is the TV on and connected to the network?"
    )


def test_list(faker: Faker, mock_config: MagicMock) -> None:
    mock_config.return_value = {
        "default_tv": faker.pystr(),
        "tvs": {faker.pystr(): {"hostname": faker.pystr(), "mac": faker.pystr()}},
    }

    result = runner.invoke(app, ["tv", "list"])

    assert result.exit_code == 0


def test_remove(faker: Faker, mock_config: MagicMock) -> None:
    name = faker.pystr()
    mock_config.return_value = {"default_tv": name, "tvs": {name: {}}}

    with patch("alga.cli_tv.config.write") as mock_write:
        result = runner.invoke(app, ["tv", "remove", name])

    assert result.exit_code == 0
    mock_write.assert_called_once_with({"default_tv": "", "tvs": {}})


def test_remove_not_found(faker: Faker, mock_config: MagicMock) -> None:
    mock_config.return_value = {"default_tv": "", "tvs": {}}
    name = faker.pystr()

    result = runner.invoke(app, ["tv", "remove", name])

    assert result.exit_code == 1


def test_rename(faker: Faker, mock_config: MagicMock) -> None:
    old_name, new_name = faker.pystr(), faker.pystr()
    mock_config.return_value = {"default_tv": old_name, "tvs": {old_name: {}}}

    with patch("alga.cli_tv.config.write") as mock_write:
        result = runner.invoke(app, ["tv", "rename", old_name, new_name])

    assert result.exit_code == 0
    mock_write.assert_called_once_with({"default_tv": new_name, "tvs": {new_name: {}}})


def test_rename_not_found(faker: Faker, mock_config: MagicMock) -> None:
    mock_config.return_value = {"default_tv": "", "tvs": {}}
    old_name, new_name = faker.pystr(), faker.pystr()

    result = runner.invoke(app, ["tv", "rename", old_name, new_name])

    assert result.exit_code == 1


def test_set_default(faker: Faker, mock_config: MagicMock) -> None:
    name = faker.pystr()
    mock_config.return_value = {"default_tv": "", "tvs": {name: {}}}

    with patch("alga.cli_tv.config.write") as mock_write:
        result = runner.invoke(app, ["tv", "set-default", name])

    assert result.exit_code == 0
    mock_write.assert_called_once_with({"default_tv": name, "tvs": {name: {}}})


def test_set_default_not_found(faker: Faker, mock_config: MagicMock) -> None:
    name = faker.pystr()
    mock_config.return_value = {"default_tv": "", "tvs": {}}

    result = runner.invoke(app, ["tv", "set-default", name])

    assert result.exit_code == 1



import json
import ssl
from unittest.mock import ANY, MagicMock, call, patch

import pytest
from faker import Faker
from typer import Exit

from alga import client
from alga.payloads import get_hello_data


def test_connect(faker: Faker) -> None:
    hostname = faker.pystr()
    timeout = faker.pyint()

    with patch("alga.client.WebSocket") as mock_websocket:
        mock_websocket

        with client.connect(hostname, timeout):
            pass

    mock_websocket.assert_has_calls(
        [
            call(sslopt={"cert_reqs": ssl.CERT_NONE}),
            call().connect(
                f"wss://{hostname}:3001/", suppress_origin=True, timeout=timeout
            ),
            call().close(),
        ]
    )


def test_do_handshake(faker: Faker) -> None:
    key = faker.pystr()
    mock_connection = MagicMock()

    mock_connection.recv.return_value = json.dumps(
        {"payload": {"client-key": faker.pystr()}}
    )

    client.do_handshake(mock_connection, key)

    mock_connection.send.assert_called_once_with(json.dumps(get_hello_data(key)))


def test_do_handshake_error(faker: Faker) -> None:
    key = faker.pystr()
    mock_connection = MagicMock()

    mock_connection.recv.return_value = json.dumps({"payload": {}})

    with pytest.raises(
        Exception, match="Something went wrong with performing a handshake"
    ):
        client.do_handshake(mock_connection, key)


def test_request_no_config(faker: Faker, mock_config: MagicMock) -> None:
    mock_config.return_value = {}

    with pytest.raises(Exit) as exc_info:
        client.request(faker.pystr())

    assert exc_info.value.exit_code == 1


def test_request_tv_id_not_in_config(faker: Faker, mock_config: MagicMock) -> None:
    mock_config.return_value = {"default_tv": faker.pystr(), "tvs": {}}

    with pytest.raises(Exit) as exc_info:
        client.request(faker.pystr())

    assert exc_info.value.exit_code == 1


def test_request_no_data(faker: Faker, mock_config: MagicMock) -> None:
    name, hostname, key = faker.pystr(), faker.pystr(), faker.pystr()
    mock_config.return_value = {
        "default_tv": name,
        "tvs": {name: {"hostname": hostname, "key": key}},
    }

    uri = faker.pystr()
    payload = {"returnValue": True} | faker.pydict()

    with patch("alga.client.connect") as mock_connect:
        mock_connect().__enter__().recv.side_effect = [
            json.dumps({"payload": {"client-key": faker.pystr()}}),
            json.dumps({"payload": payload}),
        ]

        response = client.request(uri)

        assert response == payload

        mock_connect().__enter__().send.assert_has_calls(
            [
                call(ANY),  # Handshake
                call(json.dumps({"type": "request", "uri": uri})),
            ]
        )


def test_request_with_data(faker: Faker, mock_config: MagicMock) -> None:
    name, hostname, key = faker.pystr(), faker.pystr(), faker.pystr()
    mock_config.return_value = {
        "default_tv": name,
        "tvs": {name: {"hostname": hostname, "key": key}},
    }

    uri, data = faker.pystr(), faker.pydict(allowed_types=[str, float, int])
    payload = {"returnValue": True} | faker.pydict()

    with patch("alga.client.connect") as mock_connect:
        mock_connect().__enter__().recv.side_effect = [
            json.dumps({"payload": {"client-key": faker.pystr()}}),
            json.dumps({"payload": payload}),
        ]

        response = client.request(uri, data)

        assert response == payload

        mock_connect().__enter__().send.assert_has_calls(
            [
                call(ANY),  # Handshake
                call(json.dumps({"type": "request", "uri": uri, "payload": data})),
            ]
        )



import json
from typing import Annotated, Optional

from rich import print
from typer import Argument, Typer

from alga import client


app = Typer()


@app.command()
def adhoc(path: str, data: Annotated[Optional[str], Argument()] = None) -> None:
    """Send raw request to the TV"""

    if data:
        print(client.request(path, json.loads(data)))
    else:
        print(client.request(path))



from unittest.mock import MagicMock, patch

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app
from alga.types import SoundOutputDevice


runner = CliRunner()


def test_get(faker: Faker, mock_request: MagicMock) -> None:
    sound_output = faker.pystr()
    mock_request.return_value = {"soundOutput": sound_output}

    result = runner.invoke(app, ["sound-output", "get"])

    mock_request.assert_called_once_with("ssap://audio/getSoundOutput")
    assert result.exit_code == 0
    assert result.stdout == f"The current sound output is {sound_output}\n"


def test_set(faker: Faker, mock_request: MagicMock) -> None:
    sound_output = faker.pystr()

    result = runner.invoke(app, ["sound-output", "set", sound_output])

    mock_request.assert_called_once_with(
        "ssap://audio/changeSoundOutput", {"output": sound_output}
    )
    assert result.exit_code == 0
    assert result.stdout == ""


def test_pick(faker: Faker, mock_request: MagicMock) -> None:
    sound_output_device = SoundOutputDevice(faker.pystr(), faker.pystr())

    with patch("alga.cli_sound_output.pzp") as mock_pzp:
        mock_pzp.return_value = sound_output_device

        result = runner.invoke(app, ["sound-output", "pick"])

    mock_request.assert_called_once_with(
        "ssap://audio/changeSoundOutput", {"output": sound_output_device.id_}
    )
    assert result.exit_code == 0



from dataclasses import dataclass
from typing import Any, Optional


@dataclass
class App:
    id_: str
    name: str

    def __init__(self, app: dict[str, Any]) -> None:
        self.id_ = app["id"]
        self.name = app["title"]

    def __str__(self) -> str:
        return f"{self.name} ({self.id_})"


@dataclass
class Channel:
    id_: str
    number: str
    name: str

    def __init__(self, channel: dict[str, Any]) -> None:
        self.id_ = channel["channelId"]
        self.number = channel["channelNumber"]
        self.name = channel["channelName"]

    def __str__(self) -> str:
        return f"{self.number}: {self.name}"


@dataclass
class InputDevice:
    id_: str
    name: str

    def __init__(self, input_device: dict[str, Any]) -> None:
        self.id_ = input_device["id"]
        self.name = input_device["label"]

    def __str__(self) -> str:
        return f"{self.name} ({self.id_})"


@dataclass
class SoundOutputDevice:
    id_: str
    name: str

    def __str__(self) -> str:
        return self.name


@dataclass
class State:
    tv_id: Optional[str] = None



from unittest.mock import MagicMock

from faker import Faker
from typer.testing import CliRunner

from alga.__main__ import app


runner = CliRunner()


def test_send(faker: Faker, mock_input: MagicMock) -> None:
    button = faker.pystr()

    result = runner.invoke(app, ["remote", "send", button])

    mock_input.assert_called_once_with(f"type:button\nname:{button.upper()}\n\n")
    assert result.exit_code == 0
    assert result.stdout == ""



import json
from ipaddress import ip_address
from socket import gaierror, getaddrinfo
from typing import Annotated, Optional, cast

from getmac import get_mac_address
from rich import print
from rich.console import Console
from rich.table import Table
from typer import Argument, Exit, Typer

from alga import client, config
from alga.payloads import get_hello_data


def _ip_from_hostname(hostname: str) -> Optional[str]:
    try:
        results = getaddrinfo(host=hostname, port=None)
        # TODO: Do we want to handle receiving multiple IP addresses?
        first_address = results[0]
        sockaddr = first_address[4]
        address = sockaddr[0]
        return cast(str, address)
    except gaierror:
        return None


app = Typer(no_args_is_help=True, help="Set up TVs to manage via Alga")


@app.command()
def add(
    name: Annotated[str, Argument()], hostname: Annotated[str, Argument()] = "lgwebostv"
) -> None:  # pragma: no cover
    """Pair a new TV"""

    # Check if we have been passed an IP address
    ip: Optional[str]
    try:
        ip = ip_address(hostname).compressed
    except ValueError:
        ip = _ip_from_hostname(hostname)
        if not ip:
            print(
                f"[red]Could not find any host by the name '{hostname}'.[/red] Is the TV on and connected to the network?"
            )
            raise Exit(code=1)

    with client.connect(hostname=hostname, timeout=60) as connection:
        connection.send(json.dumps(get_hello_data()))
        response = json.loads(connection.recv())
        assert response == {
            "id": "register_0",
            "payload": {"pairingType": "PROMPT", "returnValue": True},
            "type": "response",
        }, "Unexpected response received"

        console = Console()
        with console.status("Please approve the connection request on the TV now..."):
            response = json.loads(connection.recv())

    if "client-key" not in response["payload"]:
        print("[red]Setup failed![/red]")
        raise Exit(code=1)

    mac_address = get_mac_address(ip=ip)

    cfg = config.get()
    cfg["tvs"][name] = {
        "hostname": hostname,
        "key": response["payload"]["client-key"],
        "mac": mac_address,
    }

    if not cfg.get("default_tv"):
        cfg["default_tv"] = name

    config.write(cfg)

    print("TV configured, Alga is ready to use")


@app.command()
def list() -> None:
    """List current TVs"""

    cfg = config.get()

    table = Table()
    table.add_column("Default")
    table.add_column("Name")
    table.add_column("Hostname/IP")
    table.add_column("MAC address")

    for name, tv in cfg["tvs"].items():
        default = "*" if cfg["default_tv"] == name else ""
        table.add_row(default, name, tv["hostname"], tv["mac"])

    console = Console()
    console.print(table)


@app.command()
def remove(name: Annotated[str, Argument()]) -> None:
    """Remove a TV"""

    cfg = config.get()

    try:
        cfg["tvs"].pop(name)

        if cfg["default_tv"] == name:
            cfg["default_tv"] = ""

        config.write(cfg)
    except KeyError:
        print(
            f"[red]A TV with the name '{name}' was not found in the configuration[/red]"
        )
        raise Exit(code=1)


@app.command()
def rename(
    old_name: Annotated[str, Argument()], new_name: Annotated[str, Argument()]
) -> None:
    """Change the identifier for a TV"""

    cfg = config.get()

    try:
        cfg["tvs"][new_name] = cfg["tvs"].pop(old_name)

        if cfg["default_tv"] == old_name:
            cfg["default_tv"] = new_name

        config.write(cfg)
    except KeyError:
        print(
            f"[red]A TV with the name '{old_name}' was not found in the configuration[/red]"
        )
        raise Exit(code=1)


@app.command()
def set_default(name: Annotated[str, Argument()]) -> None:
    """Set the default TV"""

    cfg = config.get()

    if name in cfg["tvs"]:
        cfg["default_tv"] = name
        config.write(cfg)
    else:
        print(
            f"[red]A TV with the name '{name}' was not found in the configuration[/red]"
        )
        raise Exit(code=1)



from faker import Faker

from alga.payloads import get_hello_data


def test_get_hello_data(faker: Faker) -> None:
    client_key = faker.pystr()

    result = get_hello_data(client_key)

    assert isinstance(result, dict)
    assert result["payload"]["client-key"] == client_key



from typing import Annotated

from pzp import pzp
from rich import print
from rich.console import Console
from rich.table import Table
from typer import Argument, Typer

from alga import client
from alga.types import Channel


app = Typer(no_args_is_help=True, help="TV channels")


@app.command()
def current() -> None:
    """Get the current channel"""

    response = client.request("ssap://tv/getCurrentChannel")
    print(
        f"The current channel is [bold]{response['channelName']}[/bold] ([italic]{response['channelNumber']}[/italic])"
    )


@app.command()
def down() -> None:
    """Change channel down"""

    client.request("ssap://tv/channelDown")


@app.command()
def list() -> None:
    """List available channels"""

    response = client.request("ssap://tv/getChannelList")

    table = Table()
    table.add_column("Type")
    table.add_column("Number")
    table.add_column("Name")

    all_channels = []
    type_to_emoji = {1: "📺", 2: "📻"}
    for channel in response["channelList"]:
        # The first item is for sorting
        all_channels.append(
            [
                int(channel["channelNumber"]),
                type_to_emoji.get(channel["channelTypeId"], "❓"),
                channel["channelNumber"],
                channel["channelName"],
            ]
        )

    for row in sorted(all_channels):
        table.add_row(*row[1:])

    console = Console()
    console.print(table)


@app.command()
def pick() -> None:
    """Show picker for selecting a channel."""

    response = client.request("ssap://tv/getChannelList")
    channels = []

    for channel in response["channelList"]:
        channels.append(Channel(channel))

    channel = pzp(candidates=channels, fullscreen=False, layout="reverse")
    if channel:
        client.request("ssap://tv/openChannel", {"channelId": channel.id_})


@app.command()
def set(value: Annotated[str, Argument()]) -> None:
    """Change to specific channel"""

    if value.isnumeric():
        # If a channel number is provided, we look up the channel ID as some models require it.
        response = client.request("ssap://tv/getChannelList")

        for channel in response["channelList"]:
            if channel["channelNumber"] == value:
                value = channel["channelId"]
                break

    client.request("ssap://tv/openChannel", {"channelId": value})


@app.command()
def up() -> None:
    """Change channel up"""

    client.request("ssap://tv/channelUp")



import json
import ssl
from collections.abc import Iterator
from contextlib import contextmanager
from typing import Any, Optional, cast

from rich import print
from typer import Exit
from websocket import WebSocket

from alga import config, state
from alga.payloads import get_hello_data


@contextmanager
def connect(hostname: str, timeout: int = 10) -> Iterator[WebSocket]:
    connection = WebSocket(sslopt={"cert_reqs": ssl.CERT_NONE})
    connection.connect(f"wss://{hostname}:3001/", suppress_origin=True, timeout=timeout)  # type: ignore[no-untyped-call]

    try:
        yield connection
    finally:
        connection.close()


def do_handshake(connection: WebSocket, key: str) -> None:
    connection.send(json.dumps(get_hello_data(key)))
    response = json.loads(connection.recv())
    if "client-key" not in response["payload"]:
        raise Exception(
            f"Something went wrong with performing a handshake. Response: {response}"
        )


def request(uri: str, data: Optional[dict[str, Any]] = None) -> dict[str, Any]:
    cfg = config.get()
    tv_id = state.tv_id or cfg.get("default_tv")

    if not tv_id:
        print("[red]No connection configured, run 'alga tv add' first[/red]")
        raise Exit(code=1)

    if tv_id not in cfg["tvs"]:
        print(f"[red]'{tv_id}' was not found in the configuration[/red]")
        raise Exit(code=1)

    tv = cfg["tvs"][tv_id]

    with connect(tv["hostname"]) as connection:
        do_handshake(connection, tv["key"])

        request: dict[str, Any] = {"type": "request", "uri": uri}

        if data:
            request.update(payload=data)

        connection.send(json.dumps(request))

        raw_response = connection.recv()
        response = json.loads(raw_response)

        assert response.get("payload", {}).get("returnValue") is True
        return cast(dict[str, Any], response["payload"])



from typing import Annotated

from pzp import pzp
from rich import print
from typer import Argument, Typer

from alga import client
from alga.types import SoundOutputDevice


app = Typer(no_args_is_help=True, help="Audio output device")


@app.command()
def get() -> None:
    """Show the current output device"""

    response = client.request("ssap://audio/getSoundOutput")
    print(f"The current sound output is [bold]{response['soundOutput']}[/bold]")


@app.command()
def pick() -> None:
    """Show picker for selecting a sound output device."""

    sound_output_device = pzp(
        candidates=[
            SoundOutputDevice("tv_speaker", "TV Speaker"),
            SoundOutputDevice("external_optical", "Optical Out Device"),
            SoundOutputDevice("tv_external_speaker", "Optical Out Device + TV Speaker"),
            SoundOutputDevice("external_arc", "HDMI (ARC) Device"),
        ],
        fullscreen=False,
        layout="reverse",
    )
    if sound_output_device:
        client.request(
            "ssap://audio/changeSoundOutput", {"output": sound_output_device.id_}
        )


@app.command()
def set(value: Annotated[str, Argument()]) -> None:
    """Change the output device"""

    client.request("ssap://audio/changeSoundOutput", {"output": value})
